package booking

import (
	"errors"
	"time"

	"cloud.o-forge.io/core/oc-lib/dbs"
	"cloud.o-forge.io/core/oc-lib/logs"
	"cloud.o-forge.io/core/oc-lib/models/common/enum"
	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/tools"
)

type BookingMongoAccessor struct {
	utils.AbstractAccessor // AbstractAccessor contains the basic fields of an accessor (model, caller)
}

// New creates a new instance of the BookingMongoAccessor
func NewAccessor(request *tools.APIRequest) *BookingMongoAccessor {
	return &BookingMongoAccessor{
		AbstractAccessor: utils.AbstractAccessor{
			Logger:  logs.CreateLogger(tools.BOOKING.String()), // Create a logger with the data type
			Request: request,
			Type:    tools.BOOKING,
		},
	}
}

/*
* Nothing special here, just the basic CRUD operations
 */
func (a *BookingMongoAccessor) DeleteOne(id string) (utils.DBObject, int, error) {
	return utils.GenericDeleteOne(id, a)
}

func (a *BookingMongoAccessor) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	if set.(*Booking).State == 0 {
		return nil, 400, errors.New("state is required")
	}
	realSet := &Booking{State: set.(*Booking).State}
	return utils.GenericUpdateOne(realSet, id, a, &Booking{})
}

func (a *BookingMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	return utils.GenericStoreOne(data, a)
}

func (a *BookingMongoAccessor) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	return utils.GenericStoreOne(data, a)
}

func (a *BookingMongoAccessor) LoadOne(id string) (utils.DBObject, int, error) {
	return utils.GenericLoadOne[*Booking](id, func(d utils.DBObject) (utils.DBObject, int, error) {
		now := time.Now()
		now = now.Add(time.Second * -60)
		if d.(*Booking).State == enum.DRAFT && now.UTC().After(d.(*Booking).ExpectedStartDate) {
			return utils.GenericDeleteOne(d.GetID(), a)
		}
		if (d.(*Booking).ExpectedEndDate) == nil {
			d.(*Booking).State = enum.FORGOTTEN
			utils.GenericRawUpdateOne(d, id, a)
		} else if d.(*Booking).State == enum.SCHEDULED && now.UTC().After(d.(*Booking).ExpectedStartDate) {
			d.(*Booking).State = enum.DELAYED
			utils.GenericRawUpdateOne(d, id, a)
		}
		return d, 200, nil
	}, a)
}

func (a *BookingMongoAccessor) LoadAll(isDraft bool) ([]utils.ShallowDBObject, int, error) {
	return utils.GenericLoadAll[*Booking](a.getExec(), isDraft, a)
}

func (a *BookingMongoAccessor) Search(filters *dbs.Filters, search string, isDraft bool) ([]utils.ShallowDBObject, int, error) {
	return utils.GenericSearch[*Booking](filters, search, (&Booking{}).GetObjectFilters(search), a.getExec(), isDraft, a)
}

func (a *BookingMongoAccessor) getExec() func(utils.DBObject) utils.ShallowDBObject {
	return func(d utils.DBObject) utils.ShallowDBObject {
		now := time.Now()
		now = now.Add(time.Second * -60)
		if d.(*Booking).State == enum.DRAFT && now.UTC().After(d.(*Booking).ExpectedStartDate) {
			utils.GenericDeleteOne(d.GetID(), a)
			return nil
		}
		if d.(*Booking).State == enum.SCHEDULED && now.UTC().After(d.(*Booking).ExpectedStartDate) {
			d.(*Booking).State = enum.DELAYED
			utils.GenericRawUpdateOne(d, d.GetID(), a)
		}
		return d
	}
}
