package compute_units

import (
	"encoding/json"
	"errors"

	"cloud.o-forge.io/core/oc-lib/dbs"
	"cloud.o-forge.io/core/oc-lib/logs"
	"cloud.o-forge.io/core/oc-lib/models/resources"
	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/tools"
)

type computeUnitsMongoAccessor struct {
	utils.AbstractAccessor // AbstractAccessor contains the basic fields of an accessor (model, caller)
}

// New creates a new instance of the computeUnitsMongoAccessor
func NewAccessor(request *tools.APIRequest) *computeUnitsMongoAccessor {
	return &computeUnitsMongoAccessor{
		AbstractAccessor: utils.AbstractAccessor{
			Logger:  logs.CreateLogger(tools.COMPUTE_UNITS.String()), // Create a logger with the data type
			Request: request,
			Type:    tools.COMPUTE_UNITS,
		},
	}
}

/*
* Nothing special here, just the basic CRUD operations
 */
func (a *computeUnitsMongoAccessor) DeleteOne(id string) (utils.DBObject, int, error) {
	return utils.GenericDeleteOne(id, a)
}

func (a *computeUnitsMongoAccessor) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	// should verify if a source is existing...
	return utils.GenericUpdateOne(set, id, a, &ComputeUnits{})
}

func (a *computeUnitsMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	return utils.GenericStoreOne(data.(*ComputeUnits), a)
}

func (a *computeUnitsMongoAccessor) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	// is a publisher... that become a resources.
	if data.IsDrafted() {
		return nil, 422, errors.New("can't publish a drafted compute units")
	}
	computeUnits := data.(*ComputeUnits)
	if computeUnits.MonitorPath == "" || computeUnits.GetID() != "" {
		return nil, 422, errors.New("publishing is only allowed is it can be monitored and be accessible")
	}
	if res, code, err := a.LoadOne(computeUnits.GetID()); err != nil {
		return nil, code, err
	} else {
		computeUnits = res.(*ComputeUnits)
	}
	resAccess := resources.NewAccessor[*resources.ComputeResource](tools.COMPUTE_RESOURCE, a.Request, func() utils.DBObject { return &resources.ComputeResource{} })
	var instance *resources.ComputeResourceInstance
	b, _ := json.Marshal(computeUnits)
	json.Unmarshal(b, instance)

	if computeUnits.ResourceID != "" {
		// TODO dependent of a existing resource
		res, code, err := resAccess.LoadOne(computeUnits.ResourceID)
		if err == nil {
			return nil, code, err
		}
		existingComputeResource := res.(*resources.ComputeResource)
		if existingComputeResource.Architecture != computeUnits.Architecture || existingComputeResource.Infrastructure != computeUnits.Infrastructure {
			return nil, 422, errors.New("should be same architecture & infrastructure from the resource")
		}
		existingComputeResource.Instances = append(existingComputeResource.Instances, instance)
		return resAccess.UpdateOne(existingComputeResource, existingComputeResource.UUID)
	} else {
		var r resources.ComputeResource
		b, _ := json.Marshal(computeUnits)
		json.Unmarshal(b, &r)
		r.Instances = append(r.Instances, instance)
		res, code, err := resAccess.StoreOne(&r)
		if err != nil {
			return nil, code, err
		}
		computeUnits.ResourceID = res.GetID()
		return a.UpdateOne(computeUnits, computeUnits.GetID())
	}
}

func (a *computeUnitsMongoAccessor) LoadOne(id string) (utils.DBObject, int, error) {
	return utils.GenericLoadOne[*ComputeUnits](id, func(d utils.DBObject) (utils.DBObject, int, error) {
		return d, 200, nil
	}, a)
}

func (a *computeUnitsMongoAccessor) LoadAll(isDraft bool) ([]utils.ShallowDBObject, int, error) {
	return utils.GenericLoadAll[*ComputeUnits](a.getExec(), isDraft, a)
}

func (a *computeUnitsMongoAccessor) Search(filters *dbs.Filters, search string, isDraft bool) ([]utils.ShallowDBObject, int, error) {
	return utils.GenericSearch[*ComputeUnits](filters, search, (&ComputeUnits{}).GetObjectFilters(search), a.getExec(), isDraft, a)
}

func (a *computeUnitsMongoAccessor) getExec() func(utils.DBObject) utils.ShallowDBObject {
	return func(d utils.DBObject) utils.ShallowDBObject {
		return d
	}
}
