package resources

import (
	"errors"
	"slices"

	"cloud.o-forge.io/core/oc-lib/dbs"
	"cloud.o-forge.io/core/oc-lib/logs"
	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/tools"
)

type ResourceMongoAccessor[T ResourceInterface] struct {
	utils.AbstractAccessor // AbstractAccessor contains the basic fields of an accessor (model, caller)
	generateData           func() utils.DBObject
}

// New creates a new instance of the computeMongoAccessor
func NewAccessor[T ResourceInterface](t tools.DataType, request *tools.APIRequest, g func() utils.DBObject) *ResourceMongoAccessor[T] {
	if !slices.Contains([]tools.DataType{tools.COMPUTE_RESOURCE, tools.STORAGE_RESOURCE, tools.PROCESSING_RESOURCE, tools.WORKFLOW_RESOURCE, tools.DATA_RESOURCE}, t) {
		return nil
	}
	return &ResourceMongoAccessor[T]{
		AbstractAccessor: utils.AbstractAccessor{
			Logger:  logs.CreateLogger(t.String()), // Create a logger with the data type
			Request: request,
			Type:    t,
		},
		generateData: g,
	}
}

/*
* Nothing special here, just the basic CRUD operations
 */
func (dca *ResourceMongoAccessor[T]) DeleteOne(id string) (utils.DBObject, int, error) {
	return utils.GenericDeleteOne(id, dca)
}

func (dca *ResourceMongoAccessor[T]) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	if dca.GetType() == tools.COMPUTE_RESOURCE {
		return nil, 404, errors.New("can't update a non existing computing units resource not reported onto compute units catalog")
	}
	set.(T).Trim()
	return utils.GenericUpdateOne(set, id, dca, dca.generateData())
}

func (dca *ResourceMongoAccessor[T]) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	if dca.GetType() == tools.COMPUTE_RESOURCE {
		return nil, 404, errors.New("can't create a non existing computing units resource not reported onto compute units catalog")
	}
	data.(T).Trim()
	return utils.GenericStoreOne(data, dca)
}

func (dca *ResourceMongoAccessor[T]) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	if dca.GetType() == tools.COMPUTE_RESOURCE {
		return nil, 404, errors.New("can't copy/publish a non existing computing units resource not reported onto compute units catalog")
	}
	return dca.StoreOne(data)
}

func (dca *ResourceMongoAccessor[T]) LoadOne(id string) (utils.DBObject, int, error) {
	return utils.GenericLoadOne[T](id, func(d utils.DBObject) (utils.DBObject, int, error) {
		d.(T).SetAllowedInstances(dca.Request)
		return d, 200, nil
	}, dca)
}

func (wfa *ResourceMongoAccessor[T]) LoadAll(isDraft bool) ([]utils.ShallowDBObject, int, error) {
	return utils.GenericLoadAll[T](func(d utils.DBObject) utils.ShallowDBObject {
		d.(T).SetAllowedInstances(wfa.Request)
		return d
	}, isDraft, wfa)
}

func (wfa *ResourceMongoAccessor[T]) Search(filters *dbs.Filters, search string, isDraft bool) ([]utils.ShallowDBObject, int, error) {
	if filters == nil && search == "*" {
		return utils.GenericLoadAll[T](func(d utils.DBObject) utils.ShallowDBObject {
			d.(T).SetAllowedInstances(wfa.Request)
			return d
		}, isDraft, wfa)
	}
	return utils.GenericSearch[T](filters, search, wfa.getResourceFilter(search),
		func(d utils.DBObject) utils.ShallowDBObject {
			d.(T).SetAllowedInstances(wfa.Request)
			return d
		}, isDraft, wfa)
}

func (abs *ResourceMongoAccessor[T]) getResourceFilter(search string) *dbs.Filters {
	return &dbs.Filters{
		Or: map[string][]dbs.Filter{ // filter by like name, short_description, description, owner, url if no filters are provided
			"abstractintanciatedresource.abstractresource.abstractobject.name":       {{Operator: dbs.LIKE.String(), Value: search}},
			"abstractintanciatedresource.abstractresource.type":                      {{Operator: dbs.LIKE.String(), Value: search}},
			"abstractintanciatedresource.abstractresource.short_description":         {{Operator: dbs.LIKE.String(), Value: search}},
			"abstractintanciatedresource.abstractresource.description":               {{Operator: dbs.LIKE.String(), Value: search}},
			"abstractintanciatedresource.abstractresource.owners.name":               {{Operator: dbs.LIKE.String(), Value: search}},
			"abstractintanciatedresource.abstractresource.abstractobject.creator_id": {{Operator: dbs.EQUAL.String(), Value: search}},
		},
	}
}
