package workflow_execution_test

import (
	"testing"
	"time"

	"cloud.o-forge.io/core/oc-lib/models/common/enum"
	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/models/workflow"
	"cloud.o-forge.io/core/oc-lib/models/workflow_execution"

	"github.com/google/uuid"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/mock"
)

type MockAccessor struct {
	mock.Mock
}

func (m *MockAccessor) LoadOne(id string) (interface{}, int, error) {
	args := m.Called(id)
	return args.Get(0), args.Int(1), args.Error(2)
}

func TestNewScheduler_ValidInput(t *testing.T) {
	s := "2025-06-16T15:00:00"
	e := "2025-06-16T17:00:00"
	dur := 7200.0
	cronStr := "0 0 * * * *"

	sched := workflow_execution.NewScheduler(s, e, dur, cronStr)

	assert.NotNil(t, sched)
	assert.Equal(t, dur, sched.DurationS)
	assert.Equal(t, cronStr, sched.Cron)
}

func TestNewScheduler_InvalidStart(t *testing.T) {
	s := "invalid"
	e := "2025-06-16T17:00:00"
	dur := 7200.0
	cronStr := "0 0 * * * *"

	sched := workflow_execution.NewScheduler(s, e, dur, cronStr)
	assert.Nil(t, sched)
}

func TestNewScheduler_InvalidEnd(t *testing.T) {
	s := "2025-06-16T15:00:00"
	e := "invalid"
	dur := 7200.0
	cronStr := "0 0 * * * *"

	sched := workflow_execution.NewScheduler(s, e, dur, cronStr)
	assert.NotNil(t, sched)
	assert.Nil(t, sched.End)
}

func TestGetDates_NoCron(t *testing.T) {
	start := time.Now()
	end := start.Add(2 * time.Hour)

	s := &workflow_execution.WorkflowSchedule{
		Start: start,
		End:   &end,
	}

	schedule, err := s.GetDates()

	assert.NoError(t, err)
	assert.Len(t, schedule, 1)
	assert.Equal(t, start, schedule[0].Start)
	assert.Equal(t, end, *schedule[0].End)
}

func TestGetDates_InvalidCron(t *testing.T) {
	start := time.Now()
	end := start.Add(2 * time.Hour)

	s := &workflow_execution.WorkflowSchedule{
		Start: start,
		End:   &end,
		Cron:  "bad cron",
	}

	_, err := s.GetDates()
	assert.Error(t, err)
}

func TestGetDates_ValidCron(t *testing.T) {
	start := time.Now()
	end := start.Add(10 * time.Minute)

	s := &workflow_execution.WorkflowSchedule{
		Start:     start,
		End:       &end,
		DurationS: 60,
		Cron:      "0 */2 * * * *",
	}

	dates, err := s.GetDates()
	assert.NoError(t, err)
	assert.Greater(t, len(dates), 0)
}

func TestGetExecutions_Success(t *testing.T) {
	start := time.Now()
	end := start.Add(1 * time.Hour)
	ws := &workflow_execution.WorkflowSchedule{
		UUID:  uuid.New().String(),
		Start: start,
		End:   &end,
	}

	wf := &workflow.Workflow{
		AbstractObject: utils.AbstractObject{
			UUID: uuid.New().String(),
			Name: "TestWorkflow",
		},
	}

	execs, err := ws.GetExecutions(wf)
	assert.NoError(t, err)
	assert.Greater(t, len(execs), 0)
	assert.Equal(t, wf.UUID, execs[0].WorkflowID)
	assert.Equal(t, ws.UUID, execs[0].ExecutionsID)
	assert.Equal(t, enum.DRAFT, execs[0].State)
}

func TestSchedules_NoRequest(t *testing.T) {
	ws := &workflow_execution.WorkflowSchedule{}

	ws, wf, execs, err := ws.Schedules("someID", nil)
	assert.Error(t, err)
	assert.Nil(t, wf)
	assert.Len(t, execs, 0)
	assert.Equal(t, ws, ws)
}

// Additional test stubs to be completed with gomock usage for:
// - CheckBooking
// - BookExecs
// - getBooking
// - Schedules (success path)
// - Planify mocking in CheckBooking
// - Peer interaction in BookExecs
// - Caller deep copy errors in getCallerCopy
// Will be continued...
